# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# Original framework from Netrender
# Original Octane script from Enrico Cerica

from math import atan, pi, degrees
import subprocess
import os
import sys
import time
import mathutils
import bpy
import octanerender

from octanerender.utils import *

ostype = sys.platform
VERSION = b"0.3"
PATH_PREFIX = "/tmp/"

def base_poll(cls, context):
	rd = context.scene.render
	return (rd.use_game_engine==False) and (rd.engine in cls.COMPAT_ENGINES)

def octane_file():
	if octanerender.octane_file != bpy.data.filepath:
		octanerender.octane_file = bpy.data.filepath
		octanerender.octane_data = True

def octane_data(octanesettings):
	octane_file()

	if octanerender.octane_data:
		octanerender.octane_data = False

class OctaneRenderButtonsPanel():
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "render"
	# COMPAT_ENGINES must be defined in each subclass, external engines can add themselves here

	@classmethod
	def poll(cls, context):
		rd = context.scene.render
		return rd.engine == 'OCT_RENDER' and rd.use_game_engine == False 

class RENDER_OCT_render_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Render"
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='SEQUENCE')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octanesettings = scene.octane_render
		layout.prop(octanesettings, "panel_mode", expand=True)

		#layout.separator()

		if octanesettings.panel_mode == "MODE_EXPORT":
			layout.operator("button_export",    icon='SCENE_DATA')
		elif octanesettings.panel_mode == "MODE_RENDER":
			layout.operator("button_render",    icon='RENDER_STILL')
		else:
			layout.operator("button_animation", icon='RENDER_ANIMATION')

			row = layout.row()
			col = row.column()
			col.prop(octanesettings, "image_output")

			row = layout.row()
			col = row.column()
			col.prop(octanesettings, "export_samples_per_image")
			col = row.column()
			col.prop(octanesettings, "export_samples_presets")
				#if scene.oct_export_samples_presets!=scene.prv_oct_export_samples_presets:
				#	scene.oct_export_samples_per_image = {
				#	  "0": 8,
				#	  "1": 32,
				#	  "2": 256,
				#	  "3": 512,
				#	  "4": 1024,
				#	  "5": 4096
				#	}[scene.oct_export_samples_presets]
					
				#	scene.prv_oct_export_samples_presets=scene.oct_export_samples_presets

class RENDER_OCT_dimensions(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Dimensions"
	COMPAT_ENGINES = {'OCT_RENDER'}

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='SEQUENCE')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octanesettings = scene.octane_render
		# Initialise resolution
		if octanesettings.resolution_width==0 or octanesettings.resolution_height==0:
			r = scene.render
			octanesettings.resolution_width = int(r.resolution_x)
			octanesettings.resolution_height = int(r.resolution_y)

		row = layout.row(align=True)
		row.menu("RENDER_MT_presets", text=bpy.types.RENDER_MT_presets.bl_label)
		#row.operator("render.preset_add", text="", icon="ZOOMIN")
		#row.operator("render.preset_add", text="", icon="ZOOMOUT").remove_active = True

		split = layout.split()

		col = split.column()
		sub = col.column(align=True)
		sub.label(text="Resolution:")
		sub.prop(octanesettings, "resolution_width", text="X")
		sub.prop(octanesettings, "resolution_height", text="Y")
		sub.prop(octanesettings, "percent_size", text="")

		col = split.column()
		sub = col.column(align=True)
		sub.label(text="Frame Range:")
		sub.prop(octanesettings, "animation_start", text="Start")
		sub.prop(octanesettings, "animation_end", text="End")
		sub.prop(octanesettings, "animation_step", text="Step")

class RENDER_OCT_project_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Project Settings"
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='URL')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octanesettings = scene.octane_render

		if octanesettings.path == "":
			layout.prop(octanesettings, "path",icon='ERROR')
		else:
			layout.prop(octanesettings, "path")

		if octanesettings.project_name == "":
			layout.prop(octanesettings, "project_name",icon='ERROR')
		else:
			layout.prop(octanesettings, "project_name")
		layout.prop(octanesettings, "unit_size")

class RENDER_OCT_environment_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Environment Settings"
	#bl_options = {'DEFAULT_CLOSED'}
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='WORLD')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octanesettings = scene.octane_render

		layout.prop(octanesettings, "replace_project")

		# Initialise with active or first camera found
		#if ocscene.active_camera=='':
		#	ocscene.oct_active_camera=ocscene.camera.name
			
		# Initialise the source light as the first sun light found
		#if octanesettings.active_light_source=='':	
		#	for lamp in [l for l in scene.objects if l.type == 'LAMP']:
		#		if lamp.data.type=="SUN":
		#			octanesettings.active_light_source=lamp.name
		#			break

		# Camera
		layout.separator()
		row = layout.row()
		row.label(text="Camera Properties", icon='SCENE')
		layout.prop(octanesettings, "export_camera")
		if octanesettings.export_camera:
			row = layout.row()
			split = row.split(percentage=0.75)
			col = split.column()
			if octanesettings.active_camera == "":
				col.prop(octanesettings, "active_camera",icon='ERROR')
			else:
				col.prop(octanesettings, "active_camera")
			col = split.column()
			col.operator("button_select_camera",icon='SCENE')

			#row = layout.row()
			layout.prop(octanesettings, "lens_aperture")
			if octanesettings.lens_aperture:
				row = layout.row()
				split = row.split(percentage=0.75)
				col = split.column()
				col.prop(octanesettings, "lens_aperture_value")
				col = split.column()
				col.prop(octanesettings, "lens_aperture_curve")

			#row = layout.row()
			layout.prop(octanesettings, "focal_depth")
			if octanesettings.focal_depth:
				row = layout.row()
				split = row.split(percentage=0.75)
				col = split.column()
				col.prop(octanesettings, "focal_depth_value")
				col = split.column()
				col.prop(octanesettings, "focal_depth_curve")

			#row = layout.row()
			layout.prop(octanesettings, "camera_motion")
			if octanesettings.camera_motion:
				row = layout.row()
				split = row.split(percentage=0.75)
				col = split.column()
				col.prop(octanesettings, "interpolate_frame")
			
		# Day light environment
		layout.separator()
		row = layout.row()
		row.label(text="Daylight Environment", icon='LAMP_SUN')
		layout.prop(octanesettings, "export_sundir")
		if octanesettings.export_sundir:
			row = layout.row()
			split = row.split(percentage=0.75)
			col = split.column()
			if octanesettings.active_light_source == "":
				col.prop(octanesettings, "active_light_source",icon='ERROR')
			else:
				col.prop(octanesettings, "active_light_source")
			col = split.column()
			col.operator("button_select_light_source", icon='LAMP_SUN')

class RENDER_OCT_export_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Export Settings"
	bl_options = {'DEFAULT_CLOSED'}
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='SCRIPTWIN')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octanesettings = scene.octane_render

		# Context
		row = layout.row()
		row.label(text=" Context",icon='GROUP')
		row = layout.row()
		split = row.split(percentage=0.50)
		col = split.column()
		col.prop(octanesettings, "export_sel_only")
		col = split.column()
		col.prop(octanesettings, "export_remove_hidden")
		
		# Output options
		layout.separator()
		row = layout.row()
		row.label(text=" Output Options",icon='MOD_MESHDEFORM')
		row = layout.row()
		split = row.split(percentage=0.33)

		col = split.column()
		col.prop(octanesettings, "export_apply_modifiers")
		col = split.column()
		col.prop(octanesettings, "export_ROTX90")
		col = split.column()
		col.prop(octanesettings, "export_copy_images")

		layout.separator()
		row = layout.row()
		row.label(text=" Export Options",icon='SETTINGS')

		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(octanesettings, "export_edges")
		col = split.column()
		col.prop(octanesettings, "export_tri")
		col = split.column()
		col.prop(octanesettings, "export_materials")
		col = split.column()
		col.prop(octanesettings, "export_UV")

		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(octanesettings, "export_normals")
		col = split.column()
		col.prop(octanesettings, "export_HQ")
		col = split.column()
		col.prop(octanesettings, "export_polygroups")
		col = split.column()
		col.prop(octanesettings, "export_curves_as_nurbs")

class RENDER_OCT_system_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane System Settings"
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='RADIO')

	def draw(self, context):
		layout = self.layout
		scene = context.scene
		octanesettings = scene.octane_render
		if octanesettings.binary == "":
			layout.prop(octanesettings, "binary",icon='ERROR')
		else:
			layout.prop(octanesettings, "binary")

		row = layout.row()
		split = row.split(percentage=0.5)
		col = split.column()
		col.prop(octanesettings, "GPU_selector")
		if octanesettings.GPU_selector:
			col = split.column()
			col.prop(octanesettings, "GPU_use_list")
		layout.prop(octanesettings, "resolution")
		layout.prop(octanesettings, "verbose")
		if octanesettings.verbose == True:
			if octanerender.Verbose == False:
				octanerender.Verbose = True
				log('Turning verbose mode on')
			octanerender.Verbose = True
		else:
			if octanerender.Verbose == True:
				log('Turning verbose mode off')
			octanerender.Verbose = False

class OctaneRenderSettings(bpy.types.IDPropertyGroup):
	pass

def addProperties():
	from bpy.props import PointerProperty, StringProperty, BoolProperty, EnumProperty, IntProperty, CollectionProperty
	bpy.types.Scene.octane_render = PointerProperty(type=OctaneRenderSettings, name="Octane Render", description="Octane Render Settings")

	default_path = os.environ.get("TEMP")
    
	if not default_path:
		if os.name == 'nt':
			default_path = "c:/tmp/"
		else:
			default_path = "/tmp/"
	elif not default_path.endswith(os.sep):
		default_path += os.sep

# Octane System settings
	OctaneRenderSettings.binary = StringProperty(
		name="Octane Binary",
		description="Requires Absolute Path",
		maxlen = 128,
		default = "",
		subtype='FILE_PATH')

# Octane Project Settings
	OctaneRenderSettings.path = StringProperty(
		name="Project Path",
		description="Requires Absolute Path",
		maxlen = 128,
		default = "",
		subtype='FILE_PATH')

	OctaneRenderSettings.project_name = StringProperty(
		name="Project name",
		description="Octane project name",
		maxlen = 128,
		default = "",
		subtype='FILE_PATH')

	OctaneRenderSettings.replace_project = BoolProperty(
		name="Overwrite project .ocs file",
		description="Will destroy material setup in Octane",
		default = False)

	OctaneRenderSettings.unit_size = EnumProperty(  
		name="Native Unit Size",
		description="Native Units",
		items=(
			("11",  "Miles", ""),
			("10", "Furlongs", ""),
			("9",   "Yards", ""),
			("8",   "Feets", ""),
			("7",   "Inches", ""),
			("6",   "Kilometers", ""),
			("5",   "Hectometers", ""),
			("4",   "Decameters", ""),
			("3",   "Meters", ""),
			("2",   "Decimeters", ""),
			("1",   "Centimeters", ""),
			("0",   "Milimeters", "")
			),
		default= "3")


	OctaneRenderSettings.panel_mode = EnumProperty(
		items=(
			("MODE_EXPORT", "Export",    "Only exports to OBJ/MTL"),
			("MODE_RENDER", "Image",     "Render still image"),
			("MODE_ANIM",   "Animation", "Render animation"),
			),
		name="Panel Mode",
		description="Render/Export target",
		default="MODE_RENDER")

# Octane Export configuration

# Film
	OctaneRenderSettings.resolution = BoolProperty(
		name="Override Octane default resolution",
		description="Export resolution",
		default = True)

	OctaneRenderSettings.resolution_width = IntProperty(
		attr="",
		name="Width",
		description="Horizontal resolution",
		min=16,
		max=4096,
		default=1024)

	OctaneRenderSettings.resolution_height = IntProperty(
		attr="",
		name="Height",
		description="Vertical resolution",
		min=16,
		max=4096,
		default=512)

	OctaneRenderSettings.percent_size = IntProperty(
		name="",
		description="Resize",
		min=1,
		max=100,
		default=100,
		subtype='PERCENTAGE')

# Camera
	OctaneRenderSettings.export_camera = BoolProperty(
		name="Export Camera settings",
		description="Select camera and click button",
		default = True)

	OctaneRenderSettings.active_camera = StringProperty(
		name="Camera",
		description="Active Camera",
		maxlen = 128,
		default = "")

	OctaneRenderSettings.lens_aperture = BoolProperty(
		name="Set Lens Aperture",
		description="Lens Aperture",
		default = False)

	OctaneRenderSettings.lens_aperture_value = IntProperty(
		attr="",
		name="Value",
		description="Value",
		min=-1, max=100, default=1)

	OctaneRenderSettings.lens_aperture_curve = BoolProperty(
		name="Curve",
		description="Curve - Not implemented yet",
		default = False)

	OctaneRenderSettings.focal_depth = BoolProperty(
		name="Set Focal Depth",
		description="Focal Depth",
		default = False)

	OctaneRenderSettings.focal_depth_value = IntProperty(
		attr="",
		name="Value",
		description="Value",
		min=-1, max=100, default=5)

	OctaneRenderSettings.focal_depth_curve = BoolProperty(
		name="Curve",
		description="Curve - Not implemented yet",
		default = False)

	OctaneRenderSettings.camera_motion = BoolProperty(
		name="Camera Motion",
		description="Use camera motion for animation",
		default = False)

	OctaneRenderSettings.interpolate_frame = EnumProperty(  
		name="Interpolate",
		description="Interpolate frame",
		items=(
			("0",  "Next", ""),
			("1", "Previous", "")
			),
		default= "0")

# Day light environment

	OctaneRenderSettings.export_sundir = BoolProperty(
		name="Export Sun Direction",
		description="Export Sun Direction",
		default = False)

	OctaneRenderSettings.active_light_source = StringProperty(
		name="Sun Light",
		description="Set a Sun Light Source",
		maxlen = 128,
		default = "")

# GPU Device

	OctaneRenderSettings.GPU_selector = BoolProperty(
		name="GPU Devices to use",
		description="Custom GPU devices config",
		default = False)

	OctaneRenderSettings.GPU_use_list = StringProperty(
		name="",
		description="List of GPUs to use (ex: 0 1 2)",
		maxlen = 128,
		default = "")

# Export
# Frame
	OctaneRenderSettings.animation_mode = BoolProperty(
		name="Animation mode",
		description="Animation mode",
		default = False)

	OctaneRenderSettings.export_mode = StringProperty(
		name="Export mode",
		description="Export mode",
		maxlen = 10,
		default = "none")

# Animation

	OctaneRenderSettings.image_output = StringProperty(
		name="Animation Output",
		description="Output path for animation frames",
		maxlen = 200,
		default = default_path,
		subtype='FILE_PATH')

	OctaneRenderSettings.animation_start = IntProperty(
		attr="",
		name="Start",
		description="Start frame",
		min=1,
		max=20000,
		default=1)

	OctaneRenderSettings.animation_end = IntProperty(
		attr="",
		name="End",
		description="End frame",
		min=1,
		max=20000,
		default=250)
				
	OctaneRenderSettings.animation_step = IntProperty(attr="",
		name="Step",
		description="Step",
		min=1,
		max=100,
		default=1)
				
	OctaneRenderSettings.export_samples_per_image = IntProperty(
		attr="",
		name="Samples",
		description="Samples per image",
		min=4,
		max=64000,
		default=512)
				
	OctaneRenderSettings.export_samples_presets = EnumProperty(  
		name="Presets",
		description="Samples per image presets",
		items=(
			("0",  "Very Low (8)", ""),
			("1", "Low (32)", ""),
			("2",  "Medium (256)", ""),
			("3",  "Medium/High (512)", ""),
			("4",  "High (1024)", ""),
			("5",  "Very High (4096)", ""),
			("6",  "Too High (8192)", ""),
			("7",  "Insane (16384)", "")
			),
		default= "2")

	OctaneRenderSettings.prv_export_samples_presets = StringProperty(
		name="Previous Samples presets",
		description="Previous Samples presets",
		maxlen = 1,
		default = " ")
	
# Advanced (obj export)

	OctaneRenderSettings.export_sel_only = BoolProperty(
		name="Selection Only",
		description="Only export objects in visible selection. Else export whole scene.",
		default = False)

	OctaneRenderSettings.export_remove_hidden = BoolProperty(
		name="Remove Hidden",
		description="Remove hidden objects from export result",
		default = False)

	OctaneRenderSettings.export_apply_modifiers = BoolProperty(
		name="Apply Modifiers",
		description="Use transformed mesh data from each object. May break vert order for morph targets.",
		default = True)

	OctaneRenderSettings.export_ROTX90=BoolProperty(
		name="Rotate X90",
		description="Rotate on export so Blenders UP is translated into OBJs UP",
		default = True)

	OctaneRenderSettings.export_copy_images = BoolProperty(
		name="Copy Images",
		description="Copy image files to the export directory, never overwrite.",
		default = True)

	OctaneRenderSettings.export_edges = BoolProperty(
		name="Edges",
		description="Edges not connected to faces.",
		default = False)

	OctaneRenderSettings.export_tri=BoolProperty(
		name="Triangulate",
		description="Triangulate quads.",
		default = False)

	OctaneRenderSettings.export_materials = BoolProperty(
		name="Materials",
		description="Write a separate MTL file with the OBJ.",
		default = True)

	OctaneRenderSettings.export_UV=BoolProperty(
		name="UVs",
		description="Export texface UV coords.",
		default = True)

	OctaneRenderSettings.export_normals = BoolProperty(
		name="Normals",
		description="Export vertex normal data (Ignored on import).",
		default = True)

	OctaneRenderSettings.export_HQ = BoolProperty(
		name="HQ",
		description="Calculate high quality normals for rendering.",
		default = False)

	OctaneRenderSettings.export_polygroups = BoolProperty(
		name="Polygroups",
		description="Export vertex groups as OBJ groups (one group per face approximation).",
		default = False)

	OctaneRenderSettings.export_curves_as_nurbs = BoolProperty(
		name="Nurbs",
		description="Export 3D nurbs curves and polylines as OBJ curves, (bezier not supported).",
		default = False)

	OctaneRenderSettings.animation_mode = BoolProperty(
		name="Animation mode",
		description="Animation mode",
		default = False)

	OctaneRenderSettings.export_mode = StringProperty(
		name="Export mode",
		description="Export mode",
		maxlen = 10,
		default = "none")

	OctaneRenderSettings.verbose = BoolProperty(
		name="Verbose logging in console",
		description="Log Octane plug-in events in console",
		default = True)












    

